import 'dart:async';

import 'package:get/get.dart';
import 'package:getx_extended/extension/export.dart';

mixin CountableController on GetxController {
  RxSet<String> _tasks = RxSet();
  RxBool isProcessing = RxBool(false);

  @override
  void onInit() {
    super.onInit();
    autoDispose(
      _tasks.listen(
        (data) {
          if (isProcessing.value != data.isNotEmpty) {
            isProcessing.toggle();
          }
        },
      ),
    );
  }

  List<StreamSubscription> _streams = [];

  @override
  void dispose() {
    _streams
      ..forEach((element) => element.cancel())
      ..clear();
    super.dispose();
  }

  void autoDispose(StreamSubscription subscription) =>
      _streams.add(subscription);

  void add(
    String uuid, {
    Duration? autoComplete,
  }) {
    _tasks.add(uuid);
    if (autoComplete != null) {
      autoDispose(Future.delayed(autoComplete)
          .toStream()
          .listen((_) => complete(uuid)));
    }
  }

  void complete(String uuid) => _tasks.remove(uuid);
}
